@@ -15,15 +15,6 @@
 /* Input: main part of filename to write, namespace & tree from parser
    Output: (in files) header, skeletons, stubs, and common code.
 */
-#ifdef HAVE_INDENT_KR
-#  define INDENT "indent -npro -kr -i8 -bad -fca -sc -sob"
-#else
-#  ifdef HAVE_INDENT
-#    define INDENT "indent -npro -bad -bap -bc -sob -br -ce -cli2 -npcs -di1 -psl -i3 -lp"
-#  else
-#    define INDENT "cat"
-#  endif
-#endif
 
 void
 orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
@@ -38,7 +29,9 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   base_filename = g_basename(base_filename);
 
   if(!disable_headers) {
-    g_string_sprintf(curfile_name, INDENT " > %s.h", base_filename);
+    g_string_sprintf(curfile_name, "%s > %s.h",
+		     c_output_formatter,
+		     base_filename);
     curfile = popen(curfile_name->str, "w");
     if(!curfile) {
       g_warning("Failed to open %s for writing: %s",
@@ -53,7 +46,9 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   hdrname = g_strdup(g_basename(curfile_name->str));
 
   if(!disable_common) {
-    g_string_sprintf(curfile_name, INDENT " > %s-common.c", base_filename);
+    g_string_sprintf(curfile_name, "%s > %s-common.c",
+		     c_output_formatter,
+		     base_filename);
     curfile = popen(curfile_name->str, "w");
     if(!curfile) {
       g_warning("Failed to open %s for writing: %s",
@@ -66,7 +61,9 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   }
 
   if(!disable_stubs) {
-    g_string_sprintf(curfile_name, INDENT " > %s-stubs.c", base_filename);
+    g_string_sprintf(curfile_name, "%s > %s-stubs.c",
+		     c_output_formatter,
+		     base_filename);
     curfile = popen(curfile_name->str, "w");
     if(!curfile) {
       g_warning("Failed to open %s for writing: %s",
@@ -79,7 +76,9 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   }
 
   if(!disable_skels) {
-    g_string_sprintf(curfile_name, INDENT " > %s-skels.c", base_filename);
+    g_string_sprintf(curfile_name, "%s > %s-skels.c",
+		     c_output_formatter,
+		     base_filename);
     curfile = popen(curfile_name->str, "w");
     if(!curfile) {
       g_warning("Failed to open %s for writing: %s",
@@ -92,7 +91,9 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   }
 
   if(enable_skeleton_impl) {
-    g_string_sprintf(curfile_name, INDENT " > %s-impl.c", base_filename);
+    g_string_sprintf(curfile_name, "%s > %s-impl.c",
+		     c_output_formatter,
+		     base_filename);
     curfile = popen(curfile_name->str, "w");
     if(!curfile) {
       g_warning("Failed to open %s for writing: %s",
@@ -190,7 +191,7 @@ orbit_cbe_print_node(IDL_tree node, int indent_level)
     g_print("IDENT: %s NSQ: %s RID: \"%s\"\n",
 	    IDL_IDENT(node).str, s,
 	    IDL_IDENT_REPO_ID(node) ? IDL_IDENT_REPO_ID(node) : "");
-    free(s);
+    g_free(s);
     break;
 
   case IDLN_TYPE_DCL:
@@ -466,29 +467,29 @@ orbit_cbe_get_typename(IDL_tree tree)
     break;
   case IDLN_TYPE_STRUCT:
     retval = orbit_cbe_get_typename(IDL_TYPE_STRUCT(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_EXCEPT_DCL:
     retval = orbit_cbe_get_typename(IDL_EXCEPT_DCL(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_TYPE_ARRAY:
     retval = orbit_cbe_get_typename(IDL_TYPE_ARRAY(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_TYPE_UNION:
     retval = orbit_cbe_get_typename(IDL_TYPE_UNION(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_TYPE_ENUM:
     retval = orbit_cbe_get_typename(IDL_TYPE_ENUM(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_IDENT:
     {
       char *id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(tree), "_", 0);
       g_string_assign(tmpstr, id);
-      free(id);
+      g_free(id);
     }
     break;
   case IDLN_PARAM_DCL:
@@ -499,7 +500,7 @@ orbit_cbe_get_typename(IDL_tree tree)
     break;
   case IDLN_INTERFACE:
     retval = orbit_cbe_get_typename(IDL_INTERFACE(tree).ident);
-    g_string_assign(tmpstr, retval); free(retval); retval = NULL;
+    g_string_assign(tmpstr, retval); g_free(retval); retval = NULL;
     break;
   case IDLN_NATIVE:
     retval = "gpointer";
@@ -531,7 +532,7 @@ orbit_cbe_write_typename(FILE *of, IDL_tree tree)
 
   fprintf(of, "%s", t);
 
-  free(t);
+  g_free(t);
 }
 
 void
@@ -607,20 +608,28 @@ orbit_cbe_write_typespec(FILE *of, IDL_tree tree)
     {
       char *id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_TYPE_ENUM(tree).ident), "_", 0);
       fprintf(of, "%s", id);
-      free(id);
+      g_free(id);
     }
     break;
   case IDLN_IDENT:
     {
       char *id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(tree), "_", 0);
       fprintf(of, "%s", id);
-      free(id);
+      g_free(id);
     }
     break;
   case IDLN_TYPE_OBJECT:
     fprintf(of, "CORBA_Object");
     break;
   case IDLN_TYPE_SEQUENCE:
+    if (IDL_NODE_TYPE(tree->up) != IDLN_TYPE_DCL) { /* FIXME */
+      GString *s;
+      s = IDL_tree_to_IDL_string (tree, NULL, IDLF_OUTPUT_NO_NEWLINES);
+      IDL_tree_error(tree, "Type `%s' must be a previously declared typedef", s->str);
+      IDL_tree_error(tree, "(orbit-idl limitation; to be implemented)");
+      g_string_free (s, TRUE);
+      orbit_IDL_compile_failed(FALSE);
+    }
     fprintf(of, "struct { CORBA_unsigned_long _maximum, _length;\n");
     orbit_cbe_write_typespec(of, IDL_TYPE_SEQUENCE(tree).simple_type_spec);
     fprintf(of, " *_buffer;\nCORBA_boolean _release; } \n");
@@ -720,7 +729,7 @@ orbit_cbe_write_const(FILE *of, IDL_tree tree)
       char *id;
       id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(tree), "_", 0);
       fprintf(of, "%s", id);
-      free(id);
+      g_free(id);
     }
     break;
   default:
@@ -943,6 +952,7 @@ cbe_get_typespec(IDL_tree node)
   case IDLN_TYPE_STRUCT:
   case IDLN_TYPE_UNION:
   case IDLN_EXCEPT_DCL:
+  case IDLN_FORWARD_DCL:
   case IDLN_INTERFACE:
   case IDLN_NATIVE:
   case IDLN_TYPE_TYPECODE:
@@ -957,9 +967,6 @@ cbe_get_typespec(IDL_tree node)
   case IDLN_MEMBER:
     return cbe_get_typespec(IDL_MEMBER(node).type_spec);
     break;
-  case IDLN_FORWARD_DCL:
-    return cbe_get_typespec(IDL_FORWARD_DCL(node).ident);
-    break;
   case IDLN_LIST:
   case IDLN_IDENT:
     return cbe_get_typespec(IDL_get_parent_node(node, IDLN_ANY, NULL));
