@@ -64,6 +64,7 @@ static void cbe_hdr_do_module(CBEHeaderInfo *hinfo);
 static void cbe_hdr_do_binop(CBEHeaderInfo *hinfo);
 static void cbe_hdr_do_unaryop(CBEHeaderInfo *hinfo);
 static void cbe_hdr_do_native(CBEHeaderInfo *hinfo);
+static void cbe_hdr_do_codefrag(CBEHeaderInfo *hinfo);
 
 void
 orbit_cbe_write_header(FILE *outfile, IDL_ns ns, IDL_tree tree)
@@ -236,6 +237,9 @@ orbit_cbe_hdr_process_piece(CBEHeaderInfo *hinfo)
   case IDLN_TYPE_TYPECODE:
     cbe_hdr_do_type_typecode(hinfo);
     break;
+  case IDLN_CODEFRAG:
+    cbe_hdr_do_codefrag(hinfo);
+    break;
   default:
     /* Should never get these */
     g_error("orbit_cbe_hdr_process_piece: Unhandled node type %s (%d)",
@@ -252,7 +256,7 @@ cbe_hdr_do_module(CBEHeaderInfo *hinfo)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_MODULE(hinfo->tree).ident), "_", 0);
   g_return_if_fail(id != NULL);
-  idupper = malloc(strlen(id) + 1);
+  idupper = g_malloc(strlen(id) + 1);
 
   for(i = 0; id[i]; i++)
     idupper[i] = toupper(id[i]);
@@ -270,7 +274,7 @@ cbe_hdr_do_module(CBEHeaderInfo *hinfo)
 #if 0
   fprintf(hinfo->of, "#endif /* __%s__ */\n\n", idupper);
 #endif
-  free(id); free(idupper);
+  g_free(id); g_free(idupper);
 }
 
 static void
@@ -352,7 +356,7 @@ cbe_hdr_do_ident(CBEHeaderInfo *hinfo)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(hinfo->tree), "_", 0);
   fprintf(hinfo->of, "typedef void *%s;\n", id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -371,7 +375,7 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
       fprintf(hinfo->of, "#ifndef _%s_defined\n"
 			 "#define _%s_defined\n\n", id, id);
       fprintf(hinfo->of, "typedef ");
-      free(id);
+      g_free(id);
     } else if(IDL_NODE_TYPE(IDL_LIST(curitem).data) == IDLN_TYPE_ARRAY) {
       id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_TYPE_ARRAY(IDL_LIST(curitem).data).ident), "_", 0);
 
@@ -381,7 +385,7 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
       fprintf(hinfo->of, "#ifndef _%s_defined\n"
 			 "#define _%s_defined\n\n", id, id);
       fprintf(hinfo->of, "typedef ");
-      free(id);
+      g_free(id);
     } else {
       fprintf(hinfo->of, "/* not IDLN_IDENT or IDLN_TYPE_ARRAY - check carefully */\n");
     }
@@ -408,7 +412,6 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
       fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
 
       switch(IDL_NODE_TYPE(IDL_TYPE_DCL(hinfo->tree).type_spec)) {
-
       case IDLN_TYPE_SEQUENCE:
 	fprintf(hinfo->of, "extern ");
 	orbit_cbe_write_typespec(hinfo->of, IDL_TYPE_SEQUENCE(IDL_TYPE_DCL(hinfo->tree).type_spec).simple_type_spec);
@@ -416,7 +419,6 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
 		"* CORBA_sequence_");
 	orbit_cbe_write_typename(hinfo->of, IDL_TYPE_SEQUENCE(IDL_TYPE_DCL(hinfo->tree).type_spec).simple_type_spec);
 	fprintf(hinfo->of, "_allocbuf(CORBA_unsigned_long len);\n");
-
       default:
 	fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
 	if(IDL_NODE_TYPE(IDL_LIST(curitem).data) != IDLN_TYPE_ARRAY
@@ -429,7 +431,7 @@ cbe_hdr_do_type_dcl(CBEHeaderInfo *hinfo)
 	  fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n ", id);
       }
 
-      free(id);
+      g_free(id);
 
     } else {
       child_hinfo.tree = IDL_LIST(curitem).data;
@@ -453,7 +455,7 @@ cbe_hdr_do_const_dcl(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "\n");
   fprintf(hinfo->of, "#endif /* !%s */\n\n", id);
 
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -490,7 +492,7 @@ cbe_hdr_do_except_dcl(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -514,7 +516,7 @@ cbe_hdr_do_attr_dcl_internal(CBEHeaderInfo *hinfo, IDL_tree current_interface, g
 	    /* Output the operation on this attribute */
 	    g_string_sprintf(attrname, i ? "_set_%s" : "_get_%s",
 			     IDL_IDENT(IDL_LIST(curitem).data).str);
-	    ident = IDL_ident_new(strdup(attrname->str));
+	    ident = IDL_ident_new(g_strdup(attrname->str));
 	    
 	    /* Tell the ident where our namespace node is, and request a return value
 	       if this is the _get operation */
@@ -539,7 +541,7 @@ cbe_hdr_do_attr_dcl_internal(CBEHeaderInfo *hinfo, IDL_tree current_interface, g
 		    IDL_OP_DCL(curop).parameter_dcls = IDL_list_new(
 			    IDL_param_dcl_new(IDL_PARAM_IN,
 					      IDL_ATTR_DCL(hinfo->tree).param_type_spec,
-					      IDL_ident_new(strdup("value"))));
+					      IDL_ident_new(g_strdup("value"))));
 	    }
 	    
 	    if(inherited==TRUE)
@@ -613,8 +615,8 @@ cbe_hdr_do_op_dcl(CBEHeaderInfo *hinfo)
 
   fprintf(hinfo->of, "CORBA_Environment *ev);\n");
 
-  free(id);
-  free(id2);
+  g_free(id);
+  g_free(id2);
 }
 
 /* FIXME This function should check for redefined inherited ops, it doesnt
@@ -643,8 +645,8 @@ cbe_hdr_do_inherited_op_dcl(CBEHeaderInfo *hinfo, IDL_tree current_interface)
 
   fprintf(hinfo->of, " %s_%s(%s _obj, ",
 	  id, IDL_IDENT(ident).str, id);
-  free(id);
-  free(id2);
+  g_free(id);
+  g_free(id2);
 
   for(curitem = IDL_OP_DCL(hinfo->tree).parameter_dcls; curitem; curitem = IDL_LIST(curitem).next) {
     hinfo->tree = IDL_LIST(curitem).data;
@@ -680,7 +682,8 @@ cbe_hdr_do_param_dcl(CBEHeaderInfo *hinfo)
 
   orbit_cbe_param_printptrs(hinfo->of,
 			    IDL_PARAM_DCL(hinfo->tree).param_type_spec, r);
-  fprintf(hinfo->of, " %s, ", IDL_IDENT(IDL_PARAM_DCL(hinfo->tree).simple_declarator).str);
+  fprintf(hinfo->of, " %s, ",
+	  IDL_IDENT(IDL_PARAM_DCL(hinfo->tree).simple_declarator).str);
 }
 
 static void
@@ -701,7 +704,7 @@ cbe_hdr_do_forward_dcl(CBEHeaderInfo *hinfo)
 
   fprintf(hinfo->of, "/* End forward declaration %s */\n\n", id);
 
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -826,7 +829,7 @@ cbe_hdr_do_type_enum(CBEHeaderInfo *hinfo)
 	    id,
 	    IDL_LIST(curitem).next?",":"");
 
-    free(id);
+    g_free(id);
   }
 
   fprintf(hinfo->of, "} %s;\n", enumid);
@@ -834,19 +837,28 @@ cbe_hdr_do_type_enum(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "extern const struct CORBA_TypeCode_struct TC_%s_struct;\n", enumid);
   fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", enumid, enumid);
 
-  free(enumid);
+  g_free(enumid);
 }
 
 static void
 cbe_hdr_do_type_sequence(CBEHeaderInfo *hinfo)
 {
+  if (IDL_NODE_TYPE(hinfo->tree->up) != IDLN_TYPE_DCL) { /* FIXME */
+    GString *s;
+    s = IDL_tree_to_IDL_string (hinfo->tree, NULL, IDLF_OUTPUT_NO_NEWLINES);
+    IDL_tree_error(hinfo->tree, "Type `%s' must be a previously declared typedef", s->str);
+    IDL_tree_error(hinfo->tree, "(orbit-idl limitation; to be implemented)");
+    g_string_free (s, TRUE);
+    orbit_IDL_compile_failed(FALSE);
+  }
+
   fprintf(hinfo->of, "struct {\n");
   fprintf(hinfo->of, "  CORBA_unsigned_long _maximum;\n");
   fprintf(hinfo->of, "  CORBA_unsigned_long _length;\n");
   fprintf(hinfo->of, "  ");
   orbit_cbe_write_typespec(hinfo->of, IDL_TYPE_SEQUENCE(hinfo->tree).simple_type_spec);
   fprintf(hinfo->of, "* _buffer;\n");
-  fprintf(hinfo->of, "  CORBA_boolean _release : 1; /* ORBit-specific; see page 19-14 of CORBA 2.2 spec */\n");
+  fprintf(hinfo->of, "  CORBA_boolean _release; /* ORBit-specific; see page 19-14 of CORBA 2.2 spec */\n");
   fprintf(hinfo->of, "}");
 }
 
@@ -875,7 +887,7 @@ cbe_hdr_do_type_array(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "extern %s_slice* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -907,7 +919,7 @@ cbe_hdr_do_type_struct(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "#define TC_%s (&TC_%s_struct)\n", id, id);
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings);\n", id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -947,7 +959,7 @@ cbe_hdr_do_type_union(CBEHeaderInfo *hinfo)
   fprintf(hinfo->of, "extern %s* %s__alloc(void);\n", id, id);
   fprintf(hinfo->of, "extern gpointer %s__free(gpointer mem, gpointer dat, CORBA_boolean free_strings); /* ORBit internals */\n", id);
 
-  free(id);
+  g_free(id);
 }
 
 
@@ -1098,7 +1110,7 @@ cbe_hdr_interface_print_vepv(IDL_tree node, FILE *of)
   char *id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(node).ident),
 				     "_", 0);
   fprintf(of, "  POA_%s__epv *%s_epv;\n", id, id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -1118,7 +1130,7 @@ cbe_hdr_interface_print_inheritance_ops(IDL_tree node, CBEHeaderInfo *current)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(node).ident), "_", 0);
   fprintf(current->of, "/* From interface %s */\n", id);
-  free(id);
+  g_free(id);
 
   /* Print the operations defined for this interface, but in current's
      namespace */
@@ -1167,7 +1179,7 @@ cbe_hdr_do_interface(CBEHeaderInfo *hinfo)
   if(IDL_INTERFACE(hinfo->tree).inheritance_spec != NULL) {
     fprintf(hinfo->of, "/* Begin inherited operations for %s */\n\n", id);
     subhinfo.tree = IDL_INTERFACE(hinfo->tree).inheritance_spec;
-    IDL_tree_traverse_parents(subhinfo.tree, (gpointer)cbe_hdr_interface_print_inheritance_ops, hinfo);
+    IDL_tree_traverse_parents(subhinfo.tree, (GFunc)cbe_hdr_interface_print_inheritance_ops, hinfo);
     fprintf(hinfo->of, "/* End inherited operations for %s */\n\n", id);
   }
 
@@ -1184,7 +1196,7 @@ cbe_hdr_do_interface(CBEHeaderInfo *hinfo)
 		     "#define _POA_%s__vepv_defined\n\n", id, id);
   fprintf(hinfo->of, "typedef struct {\n");
   fprintf(hinfo->of, "  PortableServer_ServantBase__epv* _base_epv;\n");
-  IDL_tree_traverse_parents(hinfo->tree, (gpointer)cbe_hdr_interface_print_vepv, hinfo->of);
+  IDL_tree_traverse_parents(hinfo->tree, (GFunc)cbe_hdr_interface_print_vepv, hinfo->of);
   fprintf(hinfo->of, "} POA_%s__vepv;\n", id);
   fprintf(hinfo->of, "#endif /* !_POA_%s__vepv_defined */\n\n", id);
 
@@ -1203,7 +1215,7 @@ cbe_hdr_do_interface(CBEHeaderInfo *hinfo)
 
   fprintf(hinfo->of, "/* End interface %s */\n", id);
     
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -1227,5 +1239,22 @@ cbe_hdr_do_native(CBEHeaderInfo *hinfo)
 			       "_", 0);  
   fprintf(hinfo->of, "typedef void *%s;\n", id);
 
-  free(id);
+  g_free(id);
+}
+
+static void
+cbe_hdr_do_codefrag(CBEHeaderInfo *hinfo)
+{
+  GSList *list;
+  for(list = IDL_CODEFRAG(hinfo->tree).lines; list;
+      list = g_slist_next(list)) {
+    if(!strncmp(list->data,
+		"#pragma include_defs",
+		sizeof("#pragma include_defs")-1)) {
+      memmove(list->data, ((char *)list->data) + sizeof("#pragma include_defs"),
+	      strlen(list->data) - sizeof("#pragma include_defs") + 1);
+      fprintf(hinfo->of, "#include <%s>\n", (char *)list->data);
+    } else
+      fprintf(hinfo->of, "%s", (char *)list->data);
+  }
 }
