@@ -25,8 +25,6 @@ typedef struct {
 
 static void orbit_cbe_skel_process_piece(CBESkelInfo *ski);
 static void cbe_skel_do_attr_dcl(CBESkelInfo *ski);
-static void cbe_skel_do_inherited_attr_dcl(CBESkelInfo *ski,
-					   IDL_tree current_interface);
 static void cbe_skel_do_binop(CBESkelInfo *ski);
 static void cbe_skel_do_boolean(CBESkelInfo *ski);
 static void cbe_skel_do_case_stmt(CBESkelInfo *ski);
@@ -89,7 +87,7 @@ orbit_cbe_write_skeletons(FILE *outfile, IDL_ns ns, IDL_tree tree,
 		   " */\n\n");
   fprintf(outfile, "#include <string.h>\n");
   fprintf(outfile, "#include \"%s\"\n", header_filename);
-  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += sizeof(x); }G_STMT_END\n\n");
+  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur = ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) + sizeof(x); }G_STMT_END\n");
 
   orbit_cbe_skel_process_piece(&ski);
 }
@@ -225,6 +223,7 @@ orbit_cbe_skel_process_piece(CBESkelInfo *ski)
     g_error("IDLN_ANY not handled");
     break;
   case IDLN_NATIVE:
+  case IDLN_CODEFRAG:
     break;
   default:
     g_warning("Unhandled node type %s",
@@ -253,7 +252,7 @@ cbe_skel_do_attr_dcl_internal(CBESkelInfo *ski, IDL_tree current_interface, gboo
 	    /* Output the operation on this attribute */
 	    g_string_sprintf(attrname, i ? "_set_%s" : "_get_%s",
 			     IDL_IDENT(IDL_LIST(curitem).data).str);
-	    ident = IDL_ident_new(strdup(attrname->str));
+	    ident = IDL_ident_new(g_strdup(attrname->str));
 	    
 	    /* Tell the ident where our namespace node is, and request a return value
 	       if this is the _get operation */
@@ -278,7 +277,7 @@ cbe_skel_do_attr_dcl_internal(CBESkelInfo *ski, IDL_tree current_interface, gboo
 		    IDL_OP_DCL(curop).parameter_dcls = IDL_list_new(
 			    IDL_param_dcl_new(IDL_PARAM_IN,
 					      IDL_ATTR_DCL(ski->tree).param_type_spec,
-					      IDL_ident_new(strdup("value"))));
+					      IDL_ident_new(g_strdup("value"))));
 	    }
 	    
 	    if(inherited==TRUE)
@@ -377,7 +376,7 @@ cbe_skel_do_except_dcl(CBESkelInfo *ski)
 
   fprintf(ski->of, "}\n");
 
-  free(id);
+  g_free(id);
   g_string_free(tmpstr, TRUE);
 
 
@@ -447,7 +446,7 @@ cbe_skel_print_skelptr(FILE *of, IDL_tree tree)
   default:
     break;
   }
-  free(id);
+  g_free(id);
 }
 
 static gint
@@ -511,7 +510,7 @@ cbe_skel_interface_add_relayer(IDL_tree intf, CBESkelInterfaceTraverseInfo *iti)
     }
   }
 
-  free(iface_id);
+  g_free(iface_id);
 }
 
 #if 0
@@ -527,7 +526,7 @@ cbe_skel_interface_print_relayer_internal(CBESkelInfo *ski, IDL_tree curitem, ch
     fprintf(ski->of, "    *epv = servant->vepv->%s_epv->%s;\n", iface_id,
       IDL_IDENT(IDL_OP_DCL(IDL_LIST(curitem).data).ident).str);
     fprintf(ski->of,
-      "    return (gpointer)skel_%s_%s;\n",
+      "    return (ORBitSkeleton)skel_%s_%s;\n",
       cur_id, IDL_IDENT(IDL_OP_DCL(IDL_LIST(curitem).data).ident).str);
     fprintf(ski->of, "  } else");
     break;
@@ -541,7 +540,7 @@ cbe_skel_interface_print_relayer_internal(CBESkelInfo *ski, IDL_tree curitem, ch
       fprintf(ski->of, "    *epv = servant->vepv->%s_epv->_get_%s;\n", iface_id,
 	IDL_IDENT(IDL_LIST(curdcl).data).str);
       fprintf(ski->of,
-	"    return (gpointer)skel_%s__get_%s;\n",
+	"    return (ORBitSkeleton)skel_%s__get_%s;\n",
 	cur_id,
 	IDL_IDENT(IDL_LIST(curdcl).data).str);
       fprintf(ski->of, "  } else");
@@ -553,7 +552,7 @@ cbe_skel_interface_print_relayer_internal(CBESkelInfo *ski, IDL_tree curitem, ch
         fprintf(ski->of, "    *epv = servant->vepv->%s_epv->_set_%s;\n", iface_id,
 	  IDL_IDENT(IDL_LIST(curdcl).data).str);
         fprintf(ski->of,
-	  "    return (gpointer)skel_%s__set_%s;\n",
+	  "    return (ORBitSkeleton)skel_%s__set_%s;\n",
 	  cur_id, IDL_IDENT(IDL_LIST(curdcl).data).str);
         fprintf(ski->of, "  } else");
       }
@@ -590,8 +589,8 @@ cbe_skel_interface_print_inheritance_relayer(IDL_tree node, CBESkelInfo *current
     cbe_skel_interface_print_relayer_internal(current, curitem, iface_id, cur_id);
   }
 
-  free(iface_id);
-  free(cur_id);
+  g_free(iface_id);
+  g_free(cur_id);
 }
 #endif
 
@@ -630,9 +629,9 @@ cbe_skel_interface_print_relayers(const CBESkelInterfaceTraverseInfo *iti)
       else
 	g_error("two ops with same name!!!!");
     } else {
-       fprintf(iti->ski->of, "*impl = servant->vepv->%s_epv->%s;\n",
+       fprintf(iti->ski->of, "*impl = (gpointer)servant->vepv->%s_epv->%s;\n",
 	       opi->iface_id, opi->opname);
-       fprintf(iti->ski->of, "return (gpointer)skel_%s_%s;\n",
+       fprintf(iti->ski->of, "return (ORBitSkeleton)skel_%s_%s;\n",
 	       opi->iface_id, opi->opname);
     }
     fprintf(iti->ski->of, "break;\n");
@@ -665,6 +664,8 @@ cbe_skel_interface_print_relayer(CBESkelInfo *ski)
   g_slist_free(iti.oplist);
 
   fprintf(ski->of, "}\n\n");
+
+  g_free(id);
 }
 
 static void
@@ -683,7 +684,7 @@ cbe_skel_interface_print_initializer(IDL_tree node,
 
   fprintf(ski->of, "obj->vepv[%s__classid] = servant->vepv->%s_epv;\n", id, id);
 
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -695,10 +696,10 @@ cbe_skel_interface_print_objref_initializer(CBESkelInfo *ski)
   fprintf(ski->of, "static void init_local_objref_%s(CORBA_Object obj, POA_%s *servant)\n", id, id);
   fprintf(ski->of, "{\n");
 
-  free(id);
+  g_free(id);
 
   IDL_tree_traverse_parents(ski->tree,
-			    (gpointer)cbe_skel_interface_print_initializer,
+			    (GFunc)cbe_skel_interface_print_initializer,
 			    ski);
 
   fprintf(ski->of, "}\n");
@@ -722,7 +723,7 @@ cbe_skel_interface_print_inheritance_ops(IDL_tree node, CBESkelInfo *current)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(node).ident), "_", 0);
   fprintf(current->of, "/* From interface %s */\n", id);
-  free(id);
+  g_free(id);
 
   /* Print the operations defined for this interface, but in current's
      namespace */
@@ -779,7 +780,7 @@ cbe_skel_do_interface(CBESkelInfo *ski)
 	  id, id);
   fprintf(ski->of, "{\n");
   fprintf(ski->of,
-	  "  static const PortableServer_ClassInfo class_info = {(gpointer)&get_skel_%s, \"%s\", (gpointer)&init_local_objref_%s};\n",
+	  "  static const PortableServer_ClassInfo class_info = {(ORBit_impl_finder)&get_skel_%s, \"%s\", (ORBit_local_objref_init)&init_local_objref_%s};\n",
 	  id, IDL_IDENT(IDL_INTERFACE(ski->tree).ident).repo_id, id);
 
   fprintf(ski->of,
@@ -790,7 +791,7 @@ cbe_skel_do_interface(CBESkelInfo *ski)
     id2 = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_LIST(curitem).data),
 				  "_", 0);
     fprintf(ski->of, "  POA_%s__init(((POA_%s *)servant), env);\n", id2, id2);
-    free(id2);
+    g_free(id2);
   }
 
   fprintf(ski->of, "  ORBIT_OBJECT_KEY(servant->_private)->class_info = (PortableServer_ClassInfo*) &class_info;\n");
@@ -814,11 +815,13 @@ cbe_skel_do_interface(CBESkelInfo *ski)
 				      "_", 0);
 	fprintf(ski->of, "  POA_%s__fini(((POA_%s *)servant), env);\n",
 		id2, id2);
-	free(id2);
+	g_free(id2);
       }
     }
   fprintf(ski->of, "  PortableServer_ServantBase__fini(servant, env);\n");
   fprintf(ski->of, "}\n\n");
+
+  g_free(id);
 }
 
 static void
@@ -850,7 +853,7 @@ cbe_skel_do_module(CBESkelInfo *ski)
   ski->tree = IDL_MODULE(ski->tree).definition_list;
   orbit_cbe_skel_process_piece(ski);
   fprintf(ski->of, "/***************** End module %s ***************/\n", id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -1004,7 +1007,7 @@ cbe_skel_do_op_dcl(CBESkelInfo *ski)
   id2 = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(curitem).ident), "_", 0);
 
   fprintf(ski->of, "\nstatic void\nskel_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, id2);
-  free(id); free(id2);
+  g_free(id); g_free(id2);
 
   cbe_skel_do_op_dcl_shared(ski);
 }
@@ -1026,7 +1029,7 @@ cbe_skel_do_inherited_op_dcl(CBESkelInfo *ski, IDL_tree current_interface)
   }
 
   fprintf(ski->of, "\nstatic void\nskel_%s_%s(POA_%s *_ORBIT_servant,\nGIOPRecvBuffer *_ORBIT_recv_buffer,\nCORBA_Environment *ev,\n", id, IDL_IDENT(ident).str, id);
-  free(id);
+  g_free(id);
 
   cbe_skel_do_op_dcl_shared(ski);
 }
@@ -1110,7 +1113,7 @@ cbe_skel_param_subfree(CBESkelInfo *ski)
 
   fprintf(ski->of, "%s__free(&%s, NULL, %s);\n",
 	  id, varname, ski->free_internal);
-  free(id);
+  g_free(id);
 }
 
 static void
