@@ -76,7 +76,7 @@ orbit_cbe_write_stubs(FILE *outfile, IDL_ns ns, IDL_tree tree,
 		   " */\n\n");
   fprintf(outfile, "#include <string.h>\n");
   fprintf(outfile, "#include \"%s\"\n\n", header_filename);
-  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += sizeof(x); }G_STMT_END\n");
+  fprintf(outfile, "#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur), sizeof(x)); GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur = ((guchar *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) + sizeof(x); }G_STMT_END\n");
 
   orbit_cbe_stub_process_piece(&sti);
 }
@@ -212,6 +212,7 @@ orbit_cbe_stub_process_piece(CBEStubInfo *sti)
     cbe_stub_do_wide_string(sti);
     break;
   case IDLN_NATIVE:
+  case IDLN_CODEFRAG:
     break;
   default:
     g_warning("%s not handled in stubs",
@@ -245,7 +246,7 @@ cbe_stub_do_attr_dcl_internal(CBEStubInfo *sti, IDL_tree current_interface, gboo
 	    /* Output the operation on this attribute */
 	    g_string_sprintf(attrname, i ? "_set_%s" : "_get_%s",
 			     IDL_IDENT(IDL_LIST(curitem).data).str);
-	    ident = IDL_ident_new(strdup(attrname->str));
+	    ident = IDL_ident_new(g_strdup(attrname->str));
 	    
 	    /* Tell the ident where our namespace node is, and request
 	       a return value if this is the _get operation */
@@ -271,7 +272,7 @@ cbe_stub_do_attr_dcl_internal(CBEStubInfo *sti, IDL_tree current_interface, gboo
 		    IDL_OP_DCL(curop).parameter_dcls = IDL_list_new(
 			    IDL_param_dcl_new(IDL_PARAM_IN,
 					      IDL_ATTR_DCL(sti->tree).param_type_spec,
-					      IDL_ident_new(strdup("value"))));
+					      IDL_ident_new(g_strdup("value"))));
 	    }
 	    
 	    if(inherited==TRUE)
@@ -386,7 +387,7 @@ cbe_stub_do_except_dcl(CBEStubInfo *sti)
 
   fprintf(sti->of, "}\n");
 
-  free(id);
+  g_free(id);
   g_string_free(tmpstr, TRUE);
 }
 
@@ -449,7 +450,7 @@ cbe_stub_interface_print_inheritance_ops(IDL_tree node, CBEStubInfo *current)
 
   id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(node).ident), "_", 0);
   fprintf(current->of, "/* From interface %s */\n", id);
-  free(id);
+  g_free(id);
 
   /* Print the operations defined for this interface, but in current's
      namespace */
@@ -488,9 +489,9 @@ cbe_stub_do_interface(CBEStubInfo *sti)
     id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(sti->tree).ident), "_", 0);
     fprintf(sti->of, "/* Begin inherited operations for %s */\n\n", id);
     substi.tree = IDL_INTERFACE(sti->tree).inheritance_spec;
-    IDL_tree_traverse_parents(substi.tree, (gpointer)cbe_stub_interface_print_inheritance_ops, sti);
+    IDL_tree_traverse_parents(substi.tree, (GFunc)cbe_stub_interface_print_inheritance_ops, sti);
     fprintf(sti->of, "/* End inherited operations for %s */\n\n", id);
-    free(id);
+    g_free(id);
   }
 
 }
@@ -525,7 +526,7 @@ cbe_stub_do_module(CBEStubInfo *sti)
   sti->tree = IDL_MODULE(sti->tree).definition_list;
   orbit_cbe_stub_process_piece(sti);
   fprintf(sti->of, "/***************** End module %s ***************/\n", id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -826,7 +827,7 @@ cbe_stub_do_op_dcl_shared(CBEStubInfo *sti)
   fprintf(sti->of, "%s((POA_%s__epv *)_obj->vepv[%s__classid])->%s(_obj->servant, ",
 	  IDL_OP_DCL(sti->tree).op_type_spec?"return ":"",
 	  id, id, IDL_IDENT(IDL_OP_DCL(sti->tree).ident).str);
-  free(id);
+  g_free(id);
   for(curitem = IDL_OP_DCL(sti->tree).parameter_dcls; curitem;
       curitem = IDL_LIST(curitem).next) {
     fprintf(sti->of, "%s, ",
@@ -939,7 +940,7 @@ cbe_stub_do_op_dcl(CBEStubInfo *sti)
   id2 = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_INTERFACE(curitem).ident), "_", 0);
   fprintf(sti->of, "\n%s(%s _obj,\n",
 	  id, id2);
-  free(id); free(id2);
+  g_free(id); g_free(id2);
 
   cbe_stub_do_op_dcl_shared(sti);
 }
@@ -966,7 +967,7 @@ cbe_stub_do_inherited_op_dcl(CBEStubInfo *sti, IDL_tree current_interface)
 
   fprintf(sti->of, "\n%s_%s(%s _obj,\n",
 	  id, IDL_IDENT(ident).str, id);
-  free(id);
+  g_free(id);
 
   cbe_stub_do_op_dcl_shared(sti);
 }
@@ -1050,7 +1051,7 @@ cbe_stub_op_retval_alloc(FILE *of, IDL_tree node, GString *tmpstr)
 
   id = orbit_cbe_get_typename(node);
   fprintf(of, "%s = %s__alloc();\n", tmpstr->str, id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -1095,7 +1096,7 @@ cbe_stub_op_param_alloc(FILE *of, IDL_tree node, GString *tmpstr)
 
   id = orbit_cbe_get_typename(IDL_PARAM_DCL(node).param_type_spec);
   fprintf(of, "%s = %s__alloc();\n", tmpstr->str, id);
-  free(id);
+  g_free(id);
 }
 
 static void
@@ -1135,7 +1136,7 @@ cbe_stub_op_param_free(FILE *of, IDL_tree node, GString *tmpstr)
       break;
     id = IDL_ns_ident_to_qstring(IDL_IDENT_TO_NS(IDL_TYPE_STRUCT(ts).ident), "_", 0);
     fprintf(of, "%s__free(%s, NULL, CORBA_TRUE);\n", id, tmpstr->str);
-    free(id);
+    g_free(id);
     break;
   case IDLN_TYPE_STRING:
     fprintf(of, "CORBA_free(%s);\n", tmpstr->str + 1);
