@@ -45,7 +45,9 @@ ORBit_find_alignment(CORBA_TypeCode tc)
 	retval = MAX(retval, ORBit_find_alignment(tc->discriminator));
     case CORBA_tk_except:
     case CORBA_tk_struct:
+#if ALIGNOF_CORBA_STRUCT > 1
 	retval = MAX(retval, ALIGNOF_CORBA_STRUCT);
+#endif
 	for(i = 0; i < tc->sub_parts; i++)
 	    retval = MAX(retval, ORBit_find_alignment(tc->subtypes[i]));
 	return retval;
@@ -101,8 +103,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 #if 0
     g_message("Marshalling a %d value from %#x to offset %d",
 	      tc->kind, (gulong)*val,
-	      GIOP_MESSAGE_BUFFER(buf)->message_header.message_size
-	      - sizeof(GIOPMessageHeader));
+	      GIOP_MESSAGE_BUFFER(buf)->message_header.message_size);
 #endif
 
     switch(tc->kind) {
@@ -111,40 +112,40 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
     case CORBA_tk_short:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_short));
-	(guchar *)*val += sizeof(CORBA_short);
+	*val = ((guchar *)*val) + sizeof(CORBA_short);
 	break;
     case CORBA_tk_enum:
     case CORBA_tk_long:
     case CORBA_tk_ulong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_long));
-	(guchar *)*val += sizeof(CORBA_long);
+	*val = ((guchar *)*val) + sizeof(CORBA_long);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_float));
-	(guchar *)*val += sizeof(CORBA_float);
+	*val = ((guchar *)*val) + sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_double));
-	(guchar *)*val += sizeof(CORBA_double);
+	*val = ((guchar *)*val) + sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *val, sizeof(CORBA_octet));
-	(guchar *)*val += sizeof(CORBA_octet);
+	*val = ((guchar *)*val) + sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	*val = ALIGN_ADDRESS(*val, MAX(ALIGNOF_CORBA_STRUCT, ALIGNOF_CORBA_POINTER));
 	ORBit_marshal_any(buf, *val);
-	(guchar *)*val += sizeof(CORBA_any);
+	*val = ((guchar *)*val) + sizeof(CORBA_any);
 	break;
     case CORBA_tk_TypeCode:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_encode_CORBA_TypeCode(*val, buf);
-	(guchar *)*val += sizeof(CORBA_TypeCode);
+	*val = ((guchar *)*val) + sizeof(CORBA_TypeCode);
 	break;
     case CORBA_tk_Principal:
 	*val = ALIGN_ADDRESS(*val,
@@ -157,12 +158,12 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf),
 				       *(char**)((char *)*val+sizeof(CORBA_unsigned_long)),
 				       ulval);
-	(guchar *)*val += sizeof(CORBA_Principal);
+	*val = ((guchar *)*val) + sizeof(CORBA_Principal);
 	break;
     case CORBA_tk_objref:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_marshal_object(buf, *val);
-	(guchar *)*val += sizeof(CORBA_Object);
+	*val = ((guchar *)*val) + sizeof(CORBA_Object);
 	break;
     case CORBA_tk_except:
     case CORBA_tk_struct:
@@ -194,7 +195,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 					       sizeof(CORBA_unsigned_long));
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *(char **)*val, ulval);
 	
-	(guchar *)*val += sizeof(char *);
+	*val = ((guchar *)*val) + sizeof(char *);
 	break;
     case CORBA_tk_string:
 	ulval = strlen(*(char **)*val) + 1;
@@ -206,7 +207,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 					       sizeof(CORBA_unsigned_long));
 	giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(buf), *(char **)*val, ulval);
 	
-	(guchar *)*val += sizeof(char *);
+	*val = ((guchar *)*val) + sizeof(char *);
 	break;
     case CORBA_tk_sequence:
 	{
@@ -224,7 +225,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 	    for(i = 0; i < sval->_length; i++)
 		ORBit_marshal_value(buf, &subval, tc->subtypes[0], mi);
 	    
-	    (guchar *)*val += sizeof(CORBA_sequence_octet);
+	    *val = ((guchar *)*val) + sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_array:
@@ -303,10 +304,12 @@ ORBit_get_union_tag(CORBA_TypeCode union_tc,
     discrim_val = ORBit_get_union_switch(union_tc->discriminator, val);
 
     for(i = 0; i < union_tc->sub_parts; i++) {
+	if(i == union_tc->default_index)
+	    continue;
+
 	case_val = ORBit_get_union_switch(union_tc->sublabels[i]._type,
 					  union_tc->sublabels[i]._value);
-	if(i != union_tc->default_index
-	   && case_val == discrim_val) {
+	if(case_val == discrim_val) {
 	    retval = union_tc->subtypes[i];
 	    break;
 	}
@@ -460,7 +463,7 @@ ORBit_demarshal_allocate_mem(CORBA_TypeCode tc, gint nelements)
     return retval;
 }
 
-#define DM_GET_ATOM(x, n) G_STMT_START{ GIOP_RECV_BUFFER(buf)->decoder(x, (GIOP_RECV_BUFFER(buf)->cur), n); ((char *)GIOP_RECV_BUFFER(buf)->cur) += n; }G_STMT_END
+#define DM_GET_ATOM(x, n) G_STMT_START{ GIOP_RECV_BUFFER(buf)->decoder(x, (GIOP_RECV_BUFFER(buf)->cur), n); GIOP_RECV_BUFFER(buf)->cur = ((guchar *)GIOP_RECV_BUFFER(buf)) + n; }G_STMT_END
 
 static void
 ORBit_demarshal_value(GIOPRecvBuffer *buf,
@@ -483,7 +486,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_short));
 	DM_GET_ATOM(*val, sizeof(CORBA_short));
-	(guchar *)*val += sizeof(CORBA_short);
+	*val = ((guchar *)*val) + sizeof(CORBA_short);
 	break;
     case CORBA_tk_long:
     case CORBA_tk_ulong:
@@ -491,38 +494,38 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long));
 	DM_GET_ATOM(*val, sizeof(CORBA_long));
-	(guchar *)*val += sizeof(CORBA_long);
+	*val = ((guchar *)*val) + sizeof(CORBA_long);
 	break;
     case CORBA_tk_longlong:
     case CORBA_tk_ulonglong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_LONG);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long_long));
 	DM_GET_ATOM(*val, sizeof(CORBA_long_long));
-	(guchar *)*val += sizeof(CORBA_long_long);
+	*val = ((guchar *)*val) + sizeof(CORBA_long_long);
 	break;
     case CORBA_tk_longdouble:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_DOUBLE);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_long_double));
 	DM_GET_ATOM(*val, sizeof(CORBA_long_double));
-	(guchar *)*val += sizeof(CORBA_long_double);
+	*val = ((guchar *)*val) + sizeof(CORBA_long_double);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_float));
 	DM_GET_ATOM(*val, sizeof(CORBA_float));
-	(guchar *)*val += sizeof(CORBA_float);
+	*val = ((guchar *)*val) + sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	buf->cur = ALIGN_ADDRESS(buf->cur, sizeof(CORBA_double));
 	DM_GET_ATOM(*val, sizeof(CORBA_double));
-	(guchar *)*val += sizeof(CORBA_double);
+	*val = ((guchar *)*val) + sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	DM_GET_ATOM(*val, sizeof(CORBA_octet));
-	(guchar *)*val += sizeof(CORBA_octet);
+	*val = ((guchar *)*val) + sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	{
@@ -534,14 +537,14 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    decoded = *val;
 	    decoded->_release = CORBA_FALSE;
 	    ORBit_demarshal_any(buf, decoded, dup_strings, orb);
-	    (guchar *)*val += sizeof(CORBA_any);
+	    *val = ((guchar *)*val) + sizeof(CORBA_any);
 	}
 	break;
     case CORBA_tk_TypeCode:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	ORBit_decode_CORBA_TypeCode(*val, buf);
 	CORBA_Object_duplicate(*(CORBA_Object *)*val, NULL);
-	(guchar *)*val += sizeof(CORBA_TypeCode);
+	*val = ((guchar *)*val) + sizeof(CORBA_TypeCode);
 	break;
     case CORBA_tk_Principal:
 	{
@@ -556,14 +559,14 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    DM_GET_ATOM(&p->_length, sizeof(CORBA_long));
 	    p->_buffer = ORBit_alloc(p->_length, NULL, GINT_TO_POINTER(1));
 	    memcpy(p->_buffer, buf->cur, p->_length);
-	    (char *)buf->cur += p->_length;
-	    (guchar *)*val += sizeof(CORBA_sequence_octet);
+	    buf->cur = ((guchar *)buf->cur) + p->_length;
+	    *val = ((guchar *)*val) + sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_objref:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_POINTER);
 	*(CORBA_Object *)*val = ORBit_demarshal_object(buf, orb);
-	(guchar *)*val += sizeof(CORBA_Object);
+	*val = ((guchar *)*val) + sizeof(CORBA_Object);
 	break;
     case CORBA_tk_except:
     case CORBA_tk_struct:
@@ -597,7 +600,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    *(char **)*val = CORBA_string_dup(buf->cur);
 	else
 	    *(char **)*val = buf->cur;
-	(guchar *)*val += sizeof(CORBA_char *);
+	*val = ((guchar *)*val) + sizeof(CORBA_char *);
 	break;
     case CORBA_tk_sequence:
 	{
@@ -616,7 +619,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 		   all atoms... */
 		p->_buffer = ORBit_alloc(p->_length, NULL, GINT_TO_POINTER(1));
 		memcpy(p->_buffer, buf->cur, p->_length);
-		(char *)buf->cur += p->_length;
+		buf->cur = ((guchar *)buf->cur) + p->_length;
 	    } else {
 		p->_buffer = ORBit_demarshal_allocate_mem(tc->subtypes[0],
 							  p->_length);
@@ -629,7 +632,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 					  orb);
 	    }
 
-	    (guchar *)*val += sizeof(CORBA_sequence_octet);
+	    *val = ((guchar *)*val) + sizeof(CORBA_sequence_octet);
 	}
 	break;
     case CORBA_tk_array:
@@ -694,8 +697,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_SHORT);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_SHORT);
 	*(CORBA_short *)*newval = *(CORBA_short *)*val;
-	(guchar *)*val += sizeof(CORBA_short);
-	(guchar *)*newval += sizeof(CORBA_short);
+	*val = ((guchar *)*val) + sizeof(CORBA_short);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_short);
 	break;
     case CORBA_tk_enum:
     case CORBA_tk_long:
@@ -703,44 +706,44 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG);
 	*(CORBA_long *)*newval = *(CORBA_long *)*val;
-	(guchar *)*val += sizeof(CORBA_long);
-	(guchar *)*newval += sizeof(CORBA_long);
+	*val = ((guchar *)*val) + sizeof(CORBA_long);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_long);
 	break;
     case CORBA_tk_longlong:
     case CORBA_tk_ulonglong:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_LONG);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG_LONG);
 	*(CORBA_long_long *)*newval = *(CORBA_long_long *)*val;
-	(guchar *)*val += sizeof(CORBA_long_long);
-	(guchar *)*newval += sizeof(CORBA_long_long);
+	*val = ((guchar *)*val) + sizeof(CORBA_long_long);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_long_long);
 	break;
     case CORBA_tk_longdouble:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_LONG_DOUBLE);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_LONG_DOUBLE);
 	*(CORBA_long_double *)*newval = *(CORBA_long_double *)*val;
-	(guchar *)*val += sizeof(CORBA_long_double);
-	(guchar *)*newval += sizeof(CORBA_long_double);
+	*val = ((guchar *)*val) + sizeof(CORBA_long_double);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_long_double);
 	break;
     case CORBA_tk_float:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_FLOAT);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_FLOAT);
 	*(CORBA_long *)*newval = *(CORBA_long *)*val;
-	(guchar *)*val += sizeof(CORBA_float);
-	(guchar *)*newval += sizeof(CORBA_float);
+	*val = ((guchar *)*val) + sizeof(CORBA_float);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_float);
 	break;
     case CORBA_tk_double:
 	*val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_DOUBLE);
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_DOUBLE);
 	*(CORBA_double *)*newval = *(CORBA_double *)*val;
-	(guchar *)*val += sizeof(CORBA_double);
-	(guchar *)*newval += sizeof(CORBA_double);
+	*val = ((guchar *)*val) + sizeof(CORBA_double);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_double);
 	break;
     case CORBA_tk_boolean:
     case CORBA_tk_char:
     case CORBA_tk_octet:
 	*(CORBA_octet *)*newval = *(CORBA_octet *)*val;
-	(guchar *)*val += sizeof(CORBA_octet);
-	(guchar *)*newval += sizeof(CORBA_octet);
+	*val = ((guchar *)*val) + sizeof(CORBA_octet);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_octet);
 	break;
     case CORBA_tk_any:
 	{
@@ -754,8 +757,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
                == FALSE? */
 	    newany->_value = ORBit_copy_value(oldany->_value, oldany->_type);
 	    newany->_release = CORBA_TRUE;
-	    (guchar *)*val += sizeof(CORBA_any);
-	    (guchar *)*newval += sizeof(CORBA_any);
+	    *val = ((guchar *)*val) + sizeof(CORBA_any);
+	    *newval = ((guchar *)*newval) + sizeof(CORBA_any);
 	}
 	break;
     case CORBA_tk_Principal:
@@ -773,8 +776,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	memcpy(((CORBA_Principal *)*newval)->_buffer,
 	       ((CORBA_Principal *)*val)->_buffer,
 	       ((CORBA_Principal *)*val)->_length);
-	(guchar *)*val += sizeof(CORBA_Principal);
-	(guchar *)*newval += sizeof(CORBA_Principal);
+	*val = ((guchar *)*val) + sizeof(CORBA_Principal);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_Principal);
 	break;
     case CORBA_tk_TypeCode:
     case CORBA_tk_objref:
@@ -782,8 +785,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_POINTER);
 	*(CORBA_Object *)*newval = CORBA_Object_duplicate(*(CORBA_Object *)*val,
 							  NULL);
-	(guchar *)*val += sizeof(CORBA_Object);
-	(guchar *)*newval += sizeof(CORBA_Object);
+	*val = ((guchar *)*val) + sizeof(CORBA_Object);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_Object);
 	break;
     case CORBA_tk_struct:
     case CORBA_tk_except:
@@ -812,8 +815,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	*newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_POINTER);
 	
 	*(CORBA_char **)*newval = CORBA_string_dup(*(CORBA_char **)*val);
-	(guchar *)*val += sizeof(CORBA_char *);
-	(guchar *)*newval += sizeof(CORBA_char *);
+	*val = ((guchar *)*val) + sizeof(CORBA_char *);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_char *);
 	break;
     case CORBA_tk_sequence:
 	*val = ALIGN_ADDRESS(*val,
@@ -834,8 +837,8 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	for(i = 0; i < ((CORBA_Principal *)*newval)->_length; i++) {
 	    _ORBit_copy_value(&pval1, &pval2, tc->subtypes[0]);
 	}
-	(guchar *)*val += sizeof(CORBA_sequence_octet);
-	(guchar *)*newval += sizeof(CORBA_sequence_octet);
+	*val = ((guchar *)*val) + sizeof(CORBA_sequence_octet);
+	*newval = ((guchar *)*newval) + sizeof(CORBA_sequence_octet);
 	break;
     case CORBA_tk_array:
 	for(i = 0; i < tc->length; i++) {
@@ -863,3 +866,12 @@ ORBit_copy_value(gpointer value, CORBA_TypeCode tc)
 
     return retval;
 }
+
+void
+CORBA_any__copy(CORBA_any *out, CORBA_any *in)
+{
+    out->_type = (CORBA_TypeCode)CORBA_Object_duplicate((CORBA_Object)in->_type,
+							NULL);
+    out->_value = ORBit_copy_value(in->_value, in->_type);
+    out->_release = CORBA_TRUE;
+}
