@@ -44,6 +44,10 @@ char *alloca ();
 # endif
 #endif
 
+#if defined(__STRICT_ANSI__) && defined(__GNUC__)
+#define alloca __builtin_alloca
+#endif
+
 #include <unistd.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -76,7 +80,7 @@ static void ORBit_ORB_release(CORBA_ORB orb, CORBA_Environment *ev);
 
 static const ORBit_RootObject_Interface CORBA_ORB_epv =
 {
-	(gpointer)ORBit_ORB_release
+	(void (*)(gpointer, CORBA_Environment *))ORBit_ORB_release
 };
 
 
@@ -200,8 +204,6 @@ ORBit_ORB_make_usock_connection(void)
 {
 	GIOPConnection *retval = NULL;
 	GString *tmpstr;
-	DIR *dirh;
-	struct dirent *dent;
 	struct stat statbuf;
 
 	tmpstr = g_string_new(NULL);
@@ -221,7 +223,8 @@ ORBit_ORB_make_usock_connection(void)
 				g_error ("Owner of %s is not the current user\n",
 					 tmpstr->str);
 
-			if ((statbuf.st_mode & ~(S_IRWXU|S_IFDIR)) != 0)
+			if((statbuf.st_mode & (S_IRWXG|S_IRWXO))
+			   || !S_ISDIR(statbuf.st_mode))
 				g_error ("Wrong permissions for %s\n",
 					 tmpstr->str);
 			break;
@@ -232,6 +235,7 @@ ORBit_ORB_make_usock_connection(void)
 		}
 	}
 
+#ifdef WE_DONT_CARE_ABOUT_STUPID_2DOT0DOTX_KERNELS
 	g_string_sprintf(tmpstr, "/tmp/orbit-%s",
 			 g_get_user_name());
 	dirh = opendir(tmpstr->str);
@@ -260,6 +264,7 @@ ORBit_ORB_make_usock_connection(void)
 		unlink(saddr.sun_path);
 	}
 	closedir(dirh);
+#endif /* WE_DONT_CARE_ABOUT_STUPID_2DOT0DOTX_KERNELS */
 
 	srand(time(NULL));
 	while(!retval) {
@@ -564,7 +569,7 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 	CDR_Codec *codec=data->codec, *encaps = &encaps_codec_d;
 	gboolean emit_active=data->emit_active;
 	static const CORBA_octet iiopversion[] = {1,0};
-	char codecbuf[2048];
+	CORBA_octet codecbuf[2048];
 
 	g_assert(obj!=NULL);
 	g_assert(codec!=NULL);
@@ -630,7 +635,7 @@ CORBA_char *CORBA_ORB_object_to_string(CORBA_ORB orb,
   CORBA_char *rc = NULL;
   CORBA_unsigned_long ntags;
   profile_user_data data;
-  char codecbuf[2048];
+  CORBA_octet codecbuf[2048];
   char *ctmp;
 
   g_return_val_if_fail(ev, NULL);
@@ -1089,92 +1094,6 @@ CORBA_DomainManagerList *CORBA_Object_get_domain_managers(CORBA_Object obj, CORB
 	return(NULL);
 }
 
-/* Section 7.2.2 */
-CORBA_DynAny *CORBA_ORB_create_dyn_any(CORBA_ORB obj, CORBA_any value, CORBA_Environment *ev)
-{
-	g_return_val_if_fail(ev, NULL);
-	o_return_val_if_fail(obj, NULL);
-
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynAny *CORBA_ORB_create_basic_dyn_any(CORBA_ORB obj, CORBA_TypeCode type, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynStruct *CORBA_ORB_create_dyn_struct(CORBA_ORB obj, CORBA_TypeCode type, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynSequence *CORBA_ORB_create_dyn_sequence(CORBA_ORB obj, CORBA_TypeCode type, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynArray *CORBA_ORB_create_dyn_array(CORBA_ORB obj, CORBA_TypeCode type, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynUnion *CORBA_ORB_create_dyn_union(CORBA_ORB obj,
-					   CORBA_TypeCode type,
-					   CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynEnum *CORBA_ORB_create_dyn_enum(CORBA_ORB obj,
-					 CORBA_TypeCode type,
-					 CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
-/* Section 7.2.2
- *
- * raises: InconsistentTypeCode
- */
-CORBA_DynFixed *CORBA_ORB_create_dyn_fixed(CORBA_ORB obj,
-					   CORBA_TypeCode type,
-					   CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
 CORBA_TypeCode CORBA_ORB_create_struct_tc(CORBA_ORB obj, CORBA_RepositoryId id, CORBA_Identifier name, CORBA_StructMemberSeq members, CORBA_Environment *ev)
 {
 	CORBA_TypeCode tc;
