@@ -154,103 +154,54 @@ extern void CORBA_ConstructionPolicy_make_domain_manager(
 	CORBA_boolean constr_policy,
 	CORBA_Environment *ev);
 
-extern CORBA_DynAny *CORBA_ORB_create_dyn_any(
-	CORBA_ORB obj,
-	CORBA_any value,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_struct_tc(CORBA_ORB obj,
+					  CORBA_RepositoryId id,
+					  CORBA_Identifier name,
+					  CORBA_StructMemberSeq members,
+					  CORBA_Environment *ev);
 
-extern CORBA_DynAny *CORBA_ORB_create_basic_dyn_any(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_union_tc(CORBA_ORB obj,
+					 CORBA_RepositoryId id,
+					 CORBA_Identifier name,
+					 CORBA_TypeCode discriminator_type,
+					 CORBA_UnionMemberSeq members,
+					 CORBA_Environment *ev);
 
-extern CORBA_DynStruct *CORBA_ORB_create_dyn_struct(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_enum_tc(CORBA_ORB obj,
+					CORBA_RepositoryId id,
+					CORBA_Identifier name,
+					CORBA_EnumMemberSeq members,
+					CORBA_Environment *ev);
 
-extern CORBA_DynSequence *CORBA_ORB_create_dyn_sequence(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_alias_tc(CORBA_ORB obj,
+					 CORBA_RepositoryId id,
+					 CORBA_Identifier name,
+					 CORBA_TypeCode original_type,
+					 CORBA_Environment *ev);
 
-extern CORBA_DynArray *CORBA_ORB_create_dyn_array(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_exception_tc(CORBA_ORB obj,
+					     CORBA_RepositoryId id,
+					     CORBA_Identifier name,
+					     CORBA_StructMemberSeq members,
+					     CORBA_Environment *ev);
 
-extern CORBA_DynUnion *CORBA_ORB_create_dyn_union(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_interface_tc(CORBA_ORB obj,
+					     CORBA_RepositoryId id,
+					     CORBA_Identifier name,
+					     CORBA_Environment *ev);
 
-extern CORBA_DynEnum *CORBA_ORB_create_dyn_enum(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_string_tc(CORBA_ORB obj,
+					  CORBA_unsigned_long bound,
+					  CORBA_Environment *ev);
 
-extern CORBA_DynFixed *CORBA_ORB_create_dyn_fixed(
-	CORBA_ORB obj,
-	CORBA_TypeCode type,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_wstring_tc(CORBA_ORB obj,
+					   CORBA_unsigned_long bound,
+					   CORBA_Environment *ev);
 
-extern CORBA_TypeCode CORBA_ORB_create_struct_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_StructMemberSeq members,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_union_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_TypeCode discriminator_type,
-	CORBA_UnionMemberSeq members,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_enum_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_EnumMemberSeq members,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_alias_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_TypeCode original_type,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_exception_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_StructMemberSeq members,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_interface_tc(
-	CORBA_ORB obj,
-	CORBA_RepositoryId id,
-	CORBA_Identifier name,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_string_tc(
-	CORBA_ORB obj,
-	CORBA_unsigned_long bound,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_wstring_tc(
-	CORBA_ORB obj,
-	CORBA_unsigned_long bound,
-	CORBA_Environment *ev);
-
-extern CORBA_TypeCode CORBA_ORB_create_fixed_tc(
-	CORBA_ORB obj,
-	CORBA_unsigned_short digits,
-	CORBA_short scale,
-	CORBA_Environment *ev);
+CORBA_TypeCode CORBA_ORB_create_fixed_tc(CORBA_ORB obj,
+					 CORBA_unsigned_short digits,
+					 CORBA_short scale,
+					 CORBA_Environment *ev);
 
 extern CORBA_TypeCode CORBA_ORB_create_sequence_tc(
 	CORBA_ORB obj,
