@@ -117,7 +117,7 @@ void ORBit_encode_CORBA_TypeCode(CORBA_TypeCode t, GIOPSendBuffer* buf)
 	CDR_Codec* codec = &codec_d;
 	TCEncodeContext ctx;
 	GSList* l;
-	char codecbuf[2048];
+	CORBA_octet codecbuf[2048];
 
 	CDR_codec_init_static(codec);
 
@@ -150,8 +150,9 @@ void ORBit_decode_CORBA_TypeCode(CORBA_TypeCode* t, GIOPRecvBuffer* buf)
 	codec->release_buffer=CORBA_FALSE;
 	codec->readonly=CORBA_TRUE;
 	codec->buf_len =		/* hope this is correct */
-		GIOP_MESSAGE_BUFFER(buf)->message_header.message_size +
-		(((guchar *)buf->message_body) - ((guchar *)buf->cur));
+	  ((guchar *)buf->message_body) +
+	  GIOP_MESSAGE_BUFFER(buf)->message_header.message_size
+	  - ((guchar *)buf->cur);
 		
 	codec->data_endian=GIOP_MESSAGE_BUFFER(buf)->message_header.flags & 1;
 
@@ -161,7 +162,7 @@ void ORBit_decode_CORBA_TypeCode(CORBA_TypeCode* t, GIOPRecvBuffer* buf)
 	for(l=ctx.prior_tcs;l;l=l->next)
 		g_free(l->data);
 	g_slist_free(ctx.prior_tcs);
-	((guchar *)buf->cur) += codec->rptr;
+	buf->cur = ((guchar *)buf) + codec->rptr;
 }
 	
 
@@ -174,7 +175,7 @@ static void tc_enc(CORBA_TypeCode tc,
 	TCRecursionNode* node;
 	const TkInfo* info;
 	GSList* l;
-	char codecbuf[2048];
+	CORBA_octet codecbuf[2048];
 	CDR_Codec encaps_d;
 	CDR_Codec* encaps = &encaps_d;
 
